Act as an expert Dart and Flutter engineer. Provide code optimizations, best practices, and guidance on improving Dart and Flutter projects. You are creating an open source package to help other Flutter developers easily build UI customization and design system using Mix.

You will be asked to write and review code, write tests, and to optimize performance. You will also be asked to provide advice on how to improve the code and architecture of the project.

I would like to make it easier to write tests for Attributes, which are essentially either enums, values, or DTOs (data classes), that hold a value that can be passed.

Here is some context and definitions

```dart

abstract class Attribute with Comparable, Mergeable {
  const Attribute();
}

abstract class Dto<R> with Comparable, Mergeable, Resolvable<R> {
  const Dto();
}

mixin Resolvable<T> {
  T resolve(MixData mix);
}

mixin Mergeable<T> {
  T merge(covariant T? other);

  M mergeAttr<M extends Mergeable>(M? current, M? other) {
    return current?.merge(other) ?? other;
  }
}

abstract class SingleValueAttribute<T extends SingleValueAttribute<T, R>, R>
    extends StyleAttribute<R> {
  final R value;

  const SingleValueAttribute(this.value);

  // Factory for merge methods
  // ignore: avoid-shadowing
  T create(R value);

  @override
  R resolve(MixData mix) {
    return value is Resolvable ? (value as Resolvable).resolve(mix) : value;
  }

  @override
  T merge(T? other) {
    if (other == null) return create(value);

    return value is Mergeable
        ? create((value as Mergeable).merge(other.value))
        : create(other.value);
  }

  @override
  get props => [value];
}

abstract class DtoAttribute<
    Attr extends DtoAttribute<Attr, Value, ResolvedValue>,
    Value extends Dto<ResolvedValue>,
    ResolvedValue> extends StyleAttribute<ResolvedValue> {
  final Value value;

  const DtoAttribute(this.value);

  // Factory for merge methods
  Attr create(Value value);

  @override
  ResolvedValue resolve(MixData mix) {
    return value.resolve(mix);
  }

  @override
  Attr merge(Attr? other) {
    if (other == null) return create(value);

    return create(value.merge(other.value));
  }

  @override
  get props => [value];
}

abstract class ModifiableDtoAttribute<
    Attr extends ModifiableDtoAttribute<Attr, DtoValue, ResolvedValue>,
    DtoValue extends ModifiableDto<ResolvedValue>,
    ResolvedValue> extends StyleAttribute<ResolvedValue> {
  final DtoValue value;

  const ModifiableDtoAttribute(this.value);

// Factory for merge methods
  // ignore: avoid-shadowing
  Attr create(DtoValue value);

  @override
  ResolvedValue resolve(MixData mix) {
    return value.resolve(mix);
  }

  @override
  Attr merge(Attr? other) {
    if (other == null) return create(value);

    return create((value).merge(other.value));
  }

  @override
  get props => [value];
}

abstract class StyleAttribute<T> extends Attribute with Resolvable<T> {
  const StyleAttribute();

  K? resolveAttr<K, R extends StyleAttribute<K>>(
    covariant R? resolvable,
    MixData mix,
  ) {
    R? selectedAttribute = resolvable;

    selectedAttribute ??= mix.attributeOfType<R>();

    return selectedAttribute?.resolve(mix);
  }
}

typedef ValueModifier<T> = T Function(T value);

abstract class ModifiableDto<T> extends Dto<T> {
  final T value;
  @visibleForTesting
  final ValueModifier<T>? modifier;

  const ModifiableDto(this.value, {this.modifier});

  @protected
  T modify(T valueToModify) => modifier?.call(valueToModify) ?? valueToModify;
}

abstract class MixExtension<T extends MixExtension<T>> extends ThemeExtension<T>
    with Comparable {
  const MixExtension();

  Duration lerpDuration(Duration a, Duration b, double t) {
    int lerpTicks = ((1 - t) * a.inMilliseconds + t * b.inMilliseconds).round();

    return Duration(milliseconds: lerpTicks);
  }

  int lerpInt(int a, int b, double t) {
    return ((1 - t) * a + t * b).round();
  }

  N? genericNumLerp<N extends num>(N? a, N? b, double t) {
    if (a == null && b == null) return null;
    if (a == null) return b;
    if (b == null) return a;

    return (a * (1 - t) + b * t) as N;
  }

  P snap<P>(P from, P to, double t) {
    return t < 0.5 ? from : to;
  }
}

```

Given the context above I have many classes like these, that I would like to create tests for.

```dart
class AxisAttribute extends SingleValueAttribute<AxisAttribute, Axis> {
  const AxisAttribute(super.value);

  @override
  AxisAttribute create(value) => AxisAttribute(value);
}

class MainAxisAlignmentAttribute extends SingleValueAttribute<
    MainAxisAlignmentAttribute, MainAxisAlignment> {
  const MainAxisAlignmentAttribute(super.value);

  @override
  MainAxisAlignmentAttribute create(value) => MainAxisAlignmentAttribute(value);
}

class MainAxisSizeAttribute
    extends SingleValueAttribute<MainAxisSizeAttribute, MainAxisSize> {
  const MainAxisSizeAttribute(super.value);

  @override
  MainAxisSizeAttribute create(value) => MainAxisSizeAttribute(value);
}

class CrossAxisAlignmentAttribute extends SingleValueAttribute<
    CrossAxisAlignmentAttribute, CrossAxisAlignment> {
  const CrossAxisAlignmentAttribute(super.value);

  @override
  CrossAxisAlignmentAttribute create(value) =>
      CrossAxisAlignmentAttribute(value);
}

class VerticalDirectionAttribute extends SingleValueAttribute<
    VerticalDirectionAttribute, VerticalDirection> {
  const VerticalDirectionAttribute(super.value);

  @override
  VerticalDirectionAttribute create(value) => VerticalDirectionAttribute(value);
}

```

Since they are similar, I would like to maybe have a test helper or function that abstracts some of this code so I can reuse the same funcionality.
