import 'dart:ui';

import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:mix/mix.dart';

import '../../../helpers/testing_utils.dart';

void main() {
  group('BoxSpec', () {
    test('resolve', () {
      final mix = MixData.create(
        MockBuildContext(),
        Style(
          // const AlignmentGeometryAttribute(Alignment.center),
          // const PaddingAttribute(SpacingDto.only(top: 8, bottom: 16)),
          // const MarginAttribute(SpacingDto.only(top: 10.0, bottom: 12.0)),
          // const BoxConstraintsDto(
          //   BoxConstraintsDto(maxWidth: 300.0, minHeight: 200.0),
          // ),
          // const DecorationAttribute(
          //   BoxDecorationDto(color: ColorDto(Colors.blue)),
          // ),
          // TransformAttribute(Matrix4.translationValues(10.0, 10.0, 0.0)),
          BoxSpecAttribute(
            alignment: Alignment.center,
            padding: const SpacingDto.only(top: 8, bottom: 16),
            margin: const SpacingDto.only(top: 10.0, bottom: 12.0),
            constraints:
                const BoxConstraintsDto(maxWidth: 300.0, minHeight: 200.0),
            decoration: const BoxDecorationDto(color: ColorDto(Colors.blue)),
            transform: Matrix4.translationValues(10.0, 10.0, 0.0),
            clipBehavior: Clip.antiAlias,
            width: 300,
            height: 200,
          ),
        ),
      );

      final spec = mix.attributeOf<BoxSpecAttribute>()!.resolve(mix);

      expect(spec.alignment, Alignment.center);
      expect(spec.padding, const EdgeInsets.only(bottom: 16.0, top: 8.0));
      expect(spec.margin, const EdgeInsets.only(top: 10.0, bottom: 12.0));
      expect(spec.constraints,
          const BoxConstraints(maxWidth: 300.0, minHeight: 200.0));
      expect(spec.decoration, const BoxDecoration(color: Colors.blue));

      expect(spec.transform, Matrix4.translationValues(10.0, 10.0, 0.0));
      expect(spec.clipBehavior, Clip.antiAlias);
    });

    test('copyWith', () {
      final spec = BoxSpec(
        alignment: Alignment.center,
        width: 300,
        height: 200,
        padding: const EdgeInsets.all(16.0),
        margin: const EdgeInsets.only(top: 8.0, bottom: 8.0),
        constraints: const BoxConstraints(maxWidth: 300.0, minHeight: 200.0),
        decoration: const BoxDecoration(color: Colors.blue),
        transform: Matrix4.translationValues(10.0, 10.0, 0.0),
        clipBehavior: Clip.antiAlias,
      );

      final copiedSpec = spec.copyWith(width: 250.0, height: 150.0);

      expect(copiedSpec.alignment, Alignment.center);
      expect(copiedSpec.padding, const EdgeInsets.all(16.0));
      expect(copiedSpec.margin, const EdgeInsets.only(top: 8.0, bottom: 8.0));
      expect(copiedSpec.constraints,
          const BoxConstraints(maxWidth: 300.0, minHeight: 200.0));
      expect(copiedSpec.decoration, const BoxDecoration(color: Colors.blue));

      expect(copiedSpec.transform, Matrix4.translationValues(10.0, 10.0, 0.0));
      expect(copiedSpec.clipBehavior, Clip.antiAlias);
      expect(copiedSpec.width, 250.0);
      expect(copiedSpec.height, 150.0);
    });

    test('lerp', () {
      final spec1 = BoxSpec(
        alignment: Alignment.topLeft,
        width: 300,
        height: 200,
        padding: const EdgeInsets.all(8.0),
        margin: const EdgeInsets.only(top: 4.0),
        constraints: const BoxConstraints(maxWidth: 200.0),
        decoration: const BoxDecoration(color: Colors.red),
        transform: Matrix4.identity(),
        clipBehavior: Clip.none,
      );

      final spec2 = BoxSpec(
        alignment: Alignment.bottomRight,
        width: 400,
        height: 300,
        padding: const EdgeInsets.all(16.0),
        margin: const EdgeInsets.only(top: 8.0),
        constraints: const BoxConstraints(maxWidth: 400.0),
        decoration: const BoxDecoration(color: Colors.blue),
        transform: Matrix4.rotationZ(0.5),
        clipBehavior: Clip.antiAlias,
      );

      const t = 0.5;
      final lerpedSpec = spec1.lerp(spec2, t);

      expect(lerpedSpec.alignment,
          Alignment.lerp(Alignment.topLeft, Alignment.bottomRight, t));
      expect(
          lerpedSpec.padding,
          EdgeInsets.lerp(
              const EdgeInsets.all(8.0), const EdgeInsets.all(16.0), t));
      expect(
          lerpedSpec.margin,
          EdgeInsets.lerp(const EdgeInsets.only(top: 4.0),
              const EdgeInsets.only(top: 8.0), t));
      expect(
          lerpedSpec.constraints,
          BoxConstraints.lerp(const BoxConstraints(maxWidth: 200.0),
              const BoxConstraints(maxWidth: 400.0), t));
      expect(
          lerpedSpec.decoration,
          DecorationTween(
                  begin: const BoxDecoration(color: Colors.red),
                  end: const BoxDecoration(color: Colors.blue))
              .lerp(t));

      expect(lerpedSpec.width, lerpDouble(300, 400, t));
      expect(lerpedSpec.height, lerpDouble(200, 300, t));

      expect(
          lerpedSpec.transform,
          Matrix4Tween(begin: Matrix4.identity(), end: Matrix4.rotationZ(0.5))
              .lerp(t));
      expect(lerpedSpec.clipBehavior, t < 0.5 ? Clip.none : Clip.antiAlias);
    });

    // equality
    test('equality', () {
      final spec1 = BoxSpec(
        alignment: Alignment.topLeft,
        width: 300,
        height: 200,
        padding: const EdgeInsets.all(8.0),
        margin: const EdgeInsets.only(top: 4.0),
        constraints: const BoxConstraints(maxWidth: 200.0),
        decoration: const BoxDecoration(color: Colors.red),
        transform: Matrix4.identity(),
        clipBehavior: Clip.none,
      );

      final spec2 = BoxSpec(
        alignment: Alignment.topLeft,
        width: 300,
        height: 200,
        padding: const EdgeInsets.all(8.0),
        margin: const EdgeInsets.only(top: 4.0),
        constraints: const BoxConstraints(maxWidth: 200.0),
        decoration: const BoxDecoration(color: Colors.red),
        transform: Matrix4.identity(),
        clipBehavior: Clip.none,
      );

      expect(spec1, spec2);
    });

    // merge()
    test('merge() returns correct instance', () {
      final containerSpecAttribute = BoxSpecAttribute(
        alignment: Alignment.center,
        clipBehavior: Clip.antiAlias,
        constraints: const BoxConstraintsDto(maxHeight: 100),
        decoration: const BoxDecorationDto(color: ColorDto(Colors.blue)),
        height: 100,
        margin: const SpacingDto.only(
          bottom: 10,
          left: 10,
          right: 10,
          top: 10,
        ),
        padding:
            const SpacingDto.only(bottom: 20, left: 20, right: 20, top: 20),
        transform: Matrix4.identity(),
        width: 100,
      );

      final mergedBoxSpecAttribute = containerSpecAttribute.merge(
        BoxSpecAttribute(
          alignment: Alignment.centerLeft,
          clipBehavior: Clip.antiAliasWithSaveLayer,
          constraints: const BoxConstraintsDto(maxHeight: 200),
          decoration: const BoxDecorationDto(color: ColorDto(Colors.red)),
          height: 200,
          margin: const SpacingDto.only(
            bottom: 20,
            left: 20,
            right: 20,
            top: 20,
          ),
          padding:
              const SpacingDto.only(bottom: 30, left: 30, right: 30, top: 30),
          transform: Matrix4.identity(),
          width: 200,
        ),
      );

      expect(mergedBoxSpecAttribute.alignment, Alignment.centerLeft);
      expect(mergedBoxSpecAttribute.clipBehavior, Clip.antiAliasWithSaveLayer);

      expect(mergedBoxSpecAttribute.constraints,
          const BoxConstraintsDto(maxHeight: 200));
      expect(mergedBoxSpecAttribute.decoration,
          const BoxDecorationDto(color: ColorDto(Colors.red)));
      expect(mergedBoxSpecAttribute.height, 200);
      expect(
          mergedBoxSpecAttribute.margin,
          const SpacingDto.only(
            bottom: 20,
            left: 20,
            right: 20,
            top: 20,
          ));
      expect(
          mergedBoxSpecAttribute.padding,
          const SpacingDto.only(
            bottom: 30,
            left: 30,
            right: 30,
            top: 30,
          ));
      expect(mergedBoxSpecAttribute.transform, Matrix4.identity());
      expect(mergedBoxSpecAttribute.width, 200);
    });
  });
}
